# -*- coding: utf-8 -*-
"""
Модуль расчета heapsnapshot/heaptimeline.
Файл в формате json, для расчета достаточно получить списки nodes и samples.
nodes: ["type","name","id","self_size","edge_count","trace_node_id"] - параметры
ноды из кучи.
Для расчеты нужны поля: "id" - уникальный id ноды,"self_size" - размер в памяти
на момент сохранения для heapsnapshot или на момент останова записи для
heaptimeline.
samples: ["timestamp_us","last_assigned_id"]
временные отметки на heaptimeline. timestamp_us - время в мс,
last_assigned_id - id последней созданной ноды кучи
Для heapsnapshot список samples будет пустым.
"""


import json

from sealant.errors import NoResultCalcError, NoTimeStepError
from sealant.logger import log


class HeapObject:
    """
    Класс расчета heapsnapshot/heaptimeline
    Методы класса позволяют распарсить heap файл и рассчитать размеры
    кучи в каждом шаге heaptimeline или в каждом файле heapsnapshot.
    """
    def __init__(self, heapfile):
        """
        :param heapfile: Расположение heaptimeline/heapsnapshot
        """
        self.json_file = heapfile
        self.nodes = {}
        self.samples = []
        self.result = None

    def parsing_heap_file(self):
        """
        Парсинг json файла heaptimeline/heapsnapshot.
        """
        with open(self.json_file) as file:
            timeline = json.load(file)
            nodes = timeline['nodes']
            samples = timeline['samples']
            for i in range(len(nodes[::6])):
                self.nodes[int(nodes[i * 6 + 2])] = int(nodes[i * 6 + 3])
            self.samples = [[int(samples[i * 2]), int(samples[i * 2 + 1])]
                            for i in range(len((samples[::2])))]
        return True

    def get_leak_size(self, period_dur=''):
        """
        Расчет утечки по имеющимся self.nodes и self.samples
        Для расчета с таймлайном необходимо задать period_dur
        Для таймлайна - считается объем занятой памяти в каждый шаг,
        результат возвращается списком.
        Для снэпшота - считает общий объем занятой памяти,
        результат возвращается числом.
        :param period_dur: длительность одного шага, сек
        :return: результат расчета, КБ
        """
        if self.samples:
            if len(period_dur) < 5:
                raise NoTimeStepError("Количество шагов менее 5 для таймлайна")
            log('Старт расчета heaptimeline')
            period_dur = [x * 1000000 for x in period_dur]
            result = [0]
            step = 1
            nodes_keys = (key for key in sorted(self.nodes))
            nodes_key = 0
            while nodes_key <= self.samples[0][1]:
                nodes_key = next(nodes_keys)
            for timestamp_us, last_assigned_id in self.samples:
                curr_dur = sum(period_dur[:step])
                while timestamp_us > curr_dur and not step >= len(period_dur):
                    step += 1
                    result.append(0)
                    curr_dur = sum(period_dur[:step])
                while nodes_key <= last_assigned_id:
                    if step > 0:
                        result[step-1] += self.nodes[nodes_key]
                    nodes_key = next(nodes_keys)
            self.result = [x / 1000 for x in result[1:-1]]
            log('Результат {} КБ/шаг'.format(self.result))
            if len(result) < len(period_dur):
                raise NoTimeStepError("Ошибка при записи таймлайна: результат меньше заданных шагов")
        else:
            log('Старт расчета heapsnapshot')
            heap_sum = 0
            for nodes_key in sorted(self.nodes):
                heap_sum = heap_sum + self.nodes[nodes_key]
            self.result = heap_sum / 1000
            log('Результат {} КБ'.format(self.result))
        return self.result


def check_leak_with_timeline(result, leak_size_limit):
    """
    Проверка наличия утечки в подаваемых на вход данных result.
    :param result: список с размерами кучи в каждом шаге таймлайна, КБ
    :param leak_size_limit: размер уставки для сигнализации об утечке памяти, КБ
    :return: размер утечки в КБ, наличие утечки
    """
    if len(result) < 2:
        raise NoResultCalcError('В аргументе result менее 2 значений')
    leak_size = 0
    steps = len(result)
    for i in range(int(steps*0.5) + 1):
        leak_size += sorted(result)[i]
    leak_size /= int(steps*0.5) + 1
    is_leak = True if leak_size > leak_size_limit else False
    return leak_size, is_leak


def check_leak_with_snapshots(result, leak_size_limit):
    """
    Проверка наличия утечки в подаваемых на вход данных result.
    Используется метод аппроксимации размеров кучи в снэпшотах, и определение
    приблизительного размера утечки между конечным точками полученного отрезка.
    :param result: список с размерами кучи в каждом снэпшоте, КБ
    :param leak_size_limit: размер уставки для сигнализации об утечке памяти, КБ
    :return: размер утечки в КБ, наличие утечки
    """
    if len(result) < 2:
        raise NoResultCalcError('В аргументе result менее 2 значений')
    steps = len(result)
    sum_leaks = 0
    sum_steps = 0
    sum_steps2 = 0
    leaks_steps = 0
    for i in range(steps):
        sum_leaks += result[i]
        sum_steps += i + 1
        sum_steps2 += (i + 1) ** 2
        leaks_steps += (i + 1) * result[i]
    a = steps * sum_steps2 - sum_steps ** 2
    a1 = steps * leaks_steps - sum_steps * sum_leaks
    leak_size = a1 / a
    is_leak = True if leak_size > leak_size_limit else False
    return leak_size, is_leak



# -*- coding: utf-8 -*-
"""
Exceptions
"""


class LeakError(Exception):
    """
    Найдена утечка
    """
    pass


class NoTimeStepError(Exception):
    """
    При записе таймлайна получилось менее 3 шагов
    """
    pass


class NoResultCalcError(Exception):
    """
    Нет результата для проверки утечки
    """
    pass



# -*- coding: utf-8 -*-
"""
Модуль для подключения и получения данных с ноды(Chrome/Node.js)
- снэпшотов/таймлайнов, размер DOM дерева, количевство EventListeners и др.
с использованием pyСhrome библиотеки.
Используется Chrome DevTools Protocol:
https://chromedevtools.github.io/devtools-protocol
"""


import json
import pathlib
from datetime import datetime
from time import time, sleep

import pychrome
import requests

from sealant.config import SeaLantConfig
from sealant.logger import log

conf = SeaLantConfig()


class DevToolsProtocolConnection:
    """
    Класс для подключения и обмена информацией с нодой
    """
    def __init__(self, host='', port='', ws=''):
        """
        :param host: хост для подключения к ноде
        :param port: порт для подключения к ноде
        :param ws: адрес ws:// для подключения к ноде
        """
        self.class_host = host
        self.class_port = port
        self.class_ws = ws
        self.last_response = time()   # Время последнего ответа для определения завершения сетевой активности по http
        self.requests = [set(), set()]  # [активные запросы, все вызванные методы]
        self.stats_all = [[0, 0, 0]]  # Список обновления фрагментов памяти вида [идентификатор, число объектов, размер]
        self.started = False
        self.name = 'undefined'

    def connect_to_node(self, host, port, ws_url=''):
        """
        Подключение к ноду. Если задан адрес ws - сразу подключается к нему.
        Иначе получаем адрес используя заданный хост/порт
        :param host: хост для подключения к ноде
        :param port: порт для подключения к ноде
        :param ws_url: адрес для  подключения к вебсокету ноды ws://
        """
        websocket_url = ws_url or self._websocket_debugger_url(host, port)
        self.tab = pychrome.Tab(description='mld', id='mld', type='mld',
                                webSocketDebuggerUrl=websocket_url)
        self.tab.start()
        self.started = True
        log('Подключено к ноде: {}'.format(websocket_url))
        return True

    @staticmethod
    def _websocket_debugger_url(host, port):
        """
        Получение адреса ws по хосту/порту
        :return: адрес ws
        """
        geturl = requests.get("{0}:{1}/json".format(host, port))
        return json.loads(geturl.content)[0]['webSocketDebuggerUrl']

    def disconnect_from_node(self):
        """
        Отключение от ноды и удаление подписок на события.
        При использовании стандартноо метода tab.stop валится ошибка о
        получении сообщения Null. Поэтому вручную останавливаем цикл мониторящий
        получаемые сообщения и закрываем ws.
        """
        self.tab._stopped.set()
        self.tab.wait()
        self.tab.stop()
        self.tab.del_all_listeners()
        self.started = False
        log('Отключено от ноды')
        return True

    def get_heap_file(self, timeline=True):
        """
        Функция получения снэпшота/таймлайна.
        :param timeline: True/False - heaptimeline/heapsnapshot
        :return: относительный путь загруженного файла
        """
        heap_profiler = self.tab.HeapProfiler
        heap_profiler.addHeapSnapshotChunk = self._record_heapchunks_to_file
        heap_file_type = 'heaptimeline' if timeline else 'heapsnapshot'
        path = "{0}s/{1}/".format(heap_file_type, self.name)
        pathlib.Path(path).mkdir(parents=True, exist_ok=True)
        heap_file_name = '{0}%H_%M_%S.{1}'.format(path, heap_file_type)
        self.heap_file_name = datetime.strftime(datetime.now(), heap_file_name)
        self.heap_file_out = open(self.heap_file_name, 'w')
        log('Получение ' + heap_file_type)
        if timeline:
            heap_profiler.stopTrackingHeapObjects()
        else:
            heap_profiler.takeHeapSnapshot()
        self._last_record_chunk_time = time()
        while time() - self._last_record_chunk_time < 2:
            sleep(1)
        log('Получено')
        self.heap_file_out.close()
        return self.heap_file_name

    def get_metrics(self):
        """
        :return: значения заданных в конфиге дополнительных метрик
        """
        result = []
        if conf.metrics:
            for metric_params in conf.metrics:
                metric = self.tab.Runtime.evaluate(expression=metric_params[0],
                                                   includeCommandLineAPI=True,
                                                   returnByValue=True)
                result.append(metric['result']['value'])
        return result

    def activate_wait_func(self):
        """
        Активируем домен Network и вешаем подписчиков для использования
        функции ожидания завершения загрузки.
        """
        self.tab.Network.enable()
        self.tab.Network.requestWillBeSent = self._update_sent_requests
        self.tab.Network.loadingFailed = self._update_network_responses
        self.tab.Network.loadingFinished = self._update_network_responses
        self.tab.HeapProfiler.heapStatsUpdate = self._update_memory_allocation
        return True

    def wait_full_load(self, time_after_last_resp=conf.time_after_last_resp,
                       heap_interval_min=conf.heap_interval_min,
                       heap_interval_max=conf.heap_interval_max,
                       max_heap_size=conf.max_heap_size):
        """
        Метод полного ожидания завершения действия по двум критериям.
        Критерий сети: Нет уникальных активных запросов в течение
        заданного времени (по дефолту 7 сек).
        Критерий памяти (КП): объем памяти за заданный промежуток времени
        менее заданной уставки.
        (по умолчанию за промежуток с 2 до 7 сек от текущего момента времени)
        Если ожидание более 300 сек - продолжаем тест.
        Для использования необходимо добавить:
        self.cdp.wait_full_load()
        в самом тесте после строки, в которой необходимо дождаться завершения
        загрузки (см. tests/test.py).
        :param time_after_last_resp: время с последнего уникального запроса, сек
        :param heap_interval_min: начало заданного промежутка для КП, сек
        :param heap_interval_max: конец заданного промежутка для КП, сек
        :param max_heap_size: уставка объема памяти, байт
        """
        self.last_response = time()
        start_cycle = time()
        rules = [True]
        while any(rules):
            if time() - start_cycle > 300:
                log('Не дождались завершения загрузки за 300 секунд')
                break
            start = time()
            size = 0
            size_last = 0
            for statsallnew in sorted(self.stats_all, reverse=True):
                if statsallnew[0] > start - heap_interval_min:
                    size_last += statsallnew[2]
                elif statsallnew[0] > start - heap_interval_max:
                    size += statsallnew[2]
                else:
                    break
            rules = [size_last > max_heap_size * 10, size > max_heap_size,
                     len(self.requests[0]) > 0,
                     (time() - self.last_response) < time_after_last_resp]
        self.stats_all = [[0, 0, 0]]
        self.requests = [set(), set()]
        log('Загружено')
        return True

    def twice_collect_garbage(self):
        """
        Дважды вызывает GC
        """
        self.tab.HeapProfiler.collectGarbage()
        self.tab.HeapProfiler.collectGarbage()
        log('Вызов GC')
        return True

    def _update_memory_allocation(self, **kwargs):
        """
        Обработчик, получающий и обрабатывающий обновления объемов памяти,
        занимаемых фрагментами.
        Все новые фрагменты добавляются к "self.stats_all",
        у существующих обновляется объем занимаемой памяти.
        Сообщения состоят из триплетов:
        id фрагмента, кол-во объектов во фрагменте, объем занимаемой памяти.
        """
        stats_update = kwargs['statsUpdate']
        stats = ([time(), stats_update[i], stats_update[i + 2]]
                 for i in range(0, len(stats_update), 3))
        for stat in stats:
            indexes = [indexes[1] for indexes in self.stats_all]
            if stat[1] in indexes:
                i = indexes.index(stat[1])
                self.stats_all[i][2] = stat[2]
            else:
                self.stats_all.append(stat)

    def _update_sent_requests(self, **kwargs):
        """
        Обработчик http запросов.
        Все новые запросы добавляются к "self.requests" .
        Если не задан/не найден уникальный header - то добавляется по requestId.
        """
        try:
            method_name = kwargs['request']['headers'][conf.unique_header_name]
        except KeyError:
            method_name = kwargs['requestId']
        if method_name not in self.requests[1]:
            self.requests[1].add(method_name)
            self.requests[0].add(kwargs['requestId'])

    def _update_network_responses(self, **kwargs):
        """
        Обработчик http ответов.
        При получаении ответа если id есть в запросах - он удаляется из активных
        """
        try:
            self.requests[0].remove(kwargs['requestId'])
            self.last_response = time()
        except KeyError:
            pass

    def _record_heapchunks_to_file(self, **kwargs):
        """
        Обработчик чанков снапшота/таймлайна.
        """
        self.heap_file_out.write(kwargs['chunk'])
        self._last_record_chunk_time = time()



# -*- coding: utf-8 -*-
"""
Модуль опеределения функции логгирования.
Можно в конфиге задать свою функццию логгирования
По умолчанию испоьзуется встроенная библиотека logging
"""
import logging
import sys

from sealant.config import SeaLantConfig

conf = SeaLantConfig


def log(*args, **kwargs):
    """
    Функция логгирования
    """
    return conf.logging_function(*args, **kwargs)


def set_logger():
    """
    Настраиваем логгирование. Если не задана внешняя функция - используется
    стандартная библиотека logging на уровне info
    """
    if not conf.logging_function:
        log_stream = logging.getLogger()
        log_stream.level = conf.logging_level
        stream = logging.StreamHandler(stream=sys.stdout)
        formatter = logging.Formatter('%(asctime)s %(message)s')
        stream.setFormatter(formatter)
        log_stream.addHandler(stream)
        conf.logging_function = log_stream.info
    return True



# -*- coding: utf-8 -*-


class SeaLantConfig(object):
    """
    Конфигурация.
    Можно подключить метрики, которые будут сниматься перед и после тестов
    и сравниваться между собой.
    Например, размер DOM дерева и количество EventListeners до и после тестов.
    func_metric - код для выполнения черещ домен Runtime
    metrics - кортеж с метриками
    В ответ приходит словарь, откуда достается ['result']['value']
    """

    number_of_test_repeats = 5             # Количество повторов теста в течение одной проверки (для таймлайна необходимо добавить 1 прогревочный и 1 последний повтор)
    unique_header_name = 'unique_header'   # Заголовок запроса, по которому определяется уникальность метода
    host = 'http://localhost'              # Хост для подключения к ноде
    port = '9222'                          # Порт для подключения к ноде
    websocket_url = ''                     # Адрес для  подключения к вебсокету ноды ws://
    measure_repeat = 1                     # Количество перепроверок найденной утечки
    logging_level = 20                     # 0 - NOTSET; 10 - DEBUG; 20 -INFO; 30 - WARNING; 40 - ERROR; 50 - CRITICAL
    leak_size_limit = 400                  # Порог утечки, после которого сигнализировать о ее наличии, КБ
    logging_function = None                # Внешняя функция логгирования, None - внутреннее логгирование
    default_wait_full_load = True          # Использовать функцию ожидания завершения загрузки после каждого повторения теста
                                           # Иначе - можно самостоятельно выбрать место запуска этой функции или не использовать ее
    save_leaked_heapfile = True            # Сохранение heapfile в случае нахождения утечки
    get_xml_table = True                   # Составление xml отчета в случае нахождения утечки
    path_to_save = ''                      # Путь сохранения архива с отчетом и heapfile, по умолчанию создается папка leaks в папке с тестом

    # Дополнительные метрики

    func_metric_1 = "document.getElementsByTagName('*').length"
    func_metric_2 = """
                    Array.from(document.querySelectorAll('*'))
                      .reduce(function(pre, dom){
                        var clks = getEventListeners(dom).click;
                        pre += clks ? clks.length || 0 : 0;
                        return pre
                      }, 0)
                    """
    metrics = (
        (func_metric_1, "DOM элементы"),
        (func_metric_2, "EventListeners")
    )
    cdp = None
    clear_conf_cdp = True
    time_after_last_resp = 7
    heap_interval_min = 2
    heap_interval_max = 7
    max_heap_size = 10000



# -*- coding: utf-8 -*-
"""
Модуль с основной логикой для декоратора.
Для общения с нодой(Chrome/Node.js) используется Chrome DevTools Protocol:
https://chromedevtools.github.io/devtools-protocol/
"""

import inspect
import pathlib
import shutil
import xml.etree.ElementTree as xml
from functools import wraps
from time import sleep, time

from sealant.cdp import DevToolsProtocolConnection
from sealant.config import SeaLantConfig
from sealant.errors import LeakError
from sealant.heapfile_processing import HeapObject, check_leak_with_timeline
from sealant.heapfile_processing import check_leak_with_snapshots
from sealant.logger import log, set_logger

conf = SeaLantConfig()


def sealant(timeline=True, host='', port='', ws='',
            wait_func=True):
    """
    Декорируемый объект может быть классом или методом.
    В случае класса устанавливаются параметры подключения к ноде для всех
    тестируемых методов декорируемого класса.
    В случае метода устанавливаются параметры подключения к ноде для текущего
    метода, происходит подключение к ноде, действия по нахождению
    утечки и отключение от ноды.
    Заданные хост/порт или адрес вебсокета имеют следующий приоритет в порядке
    убывания:
    1. В декораторе метода
    2. В декораторе класса
    3. В config.py
    Можно задать только хост/порт или только адрес ws. Заданный адрес ws
    имеет приоритет над хостом/портом.
    :param timeline: если False - проверка с помощью снэпшотов
    :param host: хост для подключения к ноде
    :param port: порт для подключения к ноде
    :param ws: адрес ws:// для подключения к ноде
    :param wait_func: активировать возможность использования метода cdp.wait_full_load
    """
    def wrapper(obj):
        if inspect.isclass(obj):
            return _wrapper_for_class(obj, host=host, port=port, ws=ws)
        elif inspect.isfunction(obj):
            @wraps(obj)
            def test(*args, **kwargs):
                _wrapper_for_test(obj, timeline, host, port, ws,
                                  wait_func, *args, **kwargs)
            return test
    return wrapper


def _wrapper_for_class(obj, host, port, ws):
    """
    Функция обработки класса в декораторе.
    :param obj: декорируемый класс
    :param host: хост для подключения к ноде
    :param port: порт для подключения к ноде
    :param ws: адрес ws:// для подключения к ноде
    :return: декорируемый класс с привязанным экземпляром класса
    """
    set_logger()
    obj.cdp = conf.cdp = DevToolsProtocolConnection(host=host, port=port, ws=ws)
    conf.clear_conf_cdp = False
    return obj


def _wrapper_for_test(obj, timeline, host, port, ws, wait_func,
                      *args, **kwargs):
    """
    Функция обработки теста в декораторе.
    :param obj: декорируемый тест
    :param timeline: если False - проверка с помощью снэпшотов
    :param host: хост для подключения к ноде
    :param port: порт для подключения к ноде
    :param ws: адрес ws:// для подключения к ноде
    :param wait_func: активировать возможность использования метода cdp.wait_full_load
    """
    if conf.clear_conf_cdp:
        set_logger()
        conf.cdp = DevToolsProtocolConnection(host=host, port=port,
                                              ws=ws)
    cdp = obj.cdp = conf.cdp
    cdp.name = obj.__name__
    host_name = host or cdp.class_host or conf.host
    port_num = port or cdp.class_port or conf.port
    websocket_url = ws or cdp.class_ws or conf.websocket_url or ''
    cdp.connect_to_node(host_name, port_num, websocket_url)
    cdp.tab.HeapProfiler.enable()
    if wait_func:
        cdp.activate_wait_func()
    measure_repeat = conf.measure_repeat + 1
    step_repeat = conf.number_of_test_repeats
    heap_type = 'heaptimeline' if timeline else 'heapsnapshot'
    for i in range(measure_repeat):
        log('Количество повторов: {0}/{1} '
            'Шагов: {2} '
            'Heap файл: {3}'.format(i + 1, measure_repeat,
                                    step_repeat, heap_type))
        result_metric = []
        dif_result_metrics = []
        result_metric.append(cdp.get_metrics())
        if timeline:
            result = _meas_timeline(obj, step_repeat, wait_func,
                                    *args, **kwargs)
        else:
            result = _meas_snapshot(obj, step_repeat,
                                    *args, **kwargs)
        leaksize, leak = result
        log('Leak is {:.2f} KB'.format(leaksize))
        if result_metric[0]:
            result_metric.append(cdp.get_metrics())
            for j in range(len(result_metric)):
                dif = (result_metric[1][j] - result_metric[0][j]) / step_repeat
                if dif:
                    dif_result_metrics.append([dif, conf.metrics[j][1]])
                    log("Добавлено {}/шаг: {}".format(conf.metrics[j][1], dif))
        if not leak:
            break
        step_repeat += 2
    cdp.disconnect_from_node()
    if leak:
        need_zip = False
        if conf.get_xml_table:
            _create_xml_report(cdp, leaksize, dif_result_metrics, heap_type)
            need_zip = True
        if conf.save_leaked_heapfile:
            pathlib.Path('leaks').mkdir(parents=True, exist_ok=True)
            path = "{0}s/{1}".format(heap_type, cdp.name)
            heap_file_location = '{0}leaks/{1}'.format(conf.path_to_save,
                                                       cdp.name)
            need_zip = True
        if need_zip:
            shutil.make_archive(heap_file_location, format='zip', root_dir=path)
        shutil.rmtree('{}s'.format(heap_type))
        raise LeakError("В тесте есть утечка")
    shutil.rmtree('{}s'.format(heap_type))
    return True


def _meas_timeline(decorated_function, step_repeat, wait_func,
                   *args, **kwargs):
    """
    Замер утечки с использованием таймлайна.
    Количество повторов тестируемой функции увеличивается на 2:
    добавляются прогревочный и завершающий шаги
    :param decorated_function: тестируемая функция
    :param step_repeat: количество повторов тестируемой функции
    :param args: аргументы тестируемой функции
    :param kwargs: аргументы тестируемой функции
    :return: (размер утечки в шаге в КБ, наличие утечки boolean)
    """
    cdp = conf.cdp
    cdp.tab.HeapProfiler.startTrackingHeapObjects()
    time_of_steps = []
    for i in range(step_repeat):
        start_step = time()
        sleep(0.1)
        decorated_function(*args, **kwargs)
        if conf.default_wait_full_load and wait_func:
            cdp.wait_full_load()
        cdp.twice_collect_garbage()
        time_of_steps.append(time() - start_step)
    heap_file = cdp.get_heap_file(timeline=True)
    heap_calc = HeapObject(heapfile=heap_file)
    heap_calc.parsing_heap_file()
    result = heap_calc.get_leak_size(period_dur=time_of_steps)
    return check_leak_with_timeline(result=result,
                                    leak_size_limit=conf.leak_size_limit)


def _meas_snapshot(decorated_function, step_repeat, *args, **kwargs):
    """
    Замер утечки с использованием снэпшота.
    Перед тестом два прогревочных повторая
    :param decorated_function: тестируемая функция
    :param step_repeat: количество повторов тестируемой функции
    :param args: аргументы тестируемой функции
    :param kwargs: аргументы тестируемой функции
    :return: (размер утечки в шаге в КБ, наличие утечки boolean)
    """
    cdp = conf.cdp
    heap_files = []
    results = []
    for i in range(step_repeat):
        decorated_function(*args, **kwargs)
        cdp.twice_collect_garbage()
        heap_files.append(cdp.get_heap_file(timeline=False))
    for heap_file in heap_files:
        heap_calc = HeapObject(heapfile=heap_file)
        heap_calc.parsing_heap_file()
        results.append(heap_calc.get_leak_size())
    result = check_leak_with_snapshots(result=results,
                                       leak_size_limit=conf.leak_size_limit)
    return result


def _create_xml_report(cdp, leaksize, dif_result_metrics, heap_type):
    root = xml.Element("root")
    main_report = xml.Element("LeakReport")
    root.append(main_report)
    name_report = xml.SubElement(main_report, "TestName")
    name_report.text = cdp.name
    leak_report = xml.SubElement(main_report, "LeakSize")
    leak_report.text = 'Утечка за шаг: {:.2f} KB'.format(leaksize)
    if dif_result_metrics:
        metric_report = []
        for i, dif in enumerate(dif_result_metrics):
            metric_report.append(
                xml.SubElement(main_report, 'Metric_{}'.format(i + 1)))
            metric_report[i].text = "Добавлено {0}/шаг: {1}".format(dif[1],
                                                                    dif[0])
    heap_file_report = xml.SubElement(main_report, 'HeapFile')
    heap_file_report.text = "Cохранение heapfile: {}".format(
        conf.save_leaked_heapfile)
    tree = xml.ElementTree(root)
    with open('{0}s/{1}/report.xml'.format(heap_type, cdp.name), 'wb') as fh:
        tree.write(fh, xml_declaration=True, encoding='utf-8')



from setuptools import setup

with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setup(
    name='SeaLant',
    version='1.0.2',
    packages=['sealant'],
    url='https://pypi.org/project/sealant',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='LICENSE.md',
    author='ae.udahin',
    author_email='aejudakhin@gmail.com',
    description='Библиотека с инструментом для поиска утечек памяти в процессе выполнения тестов в Chrome или Node.js',
    install_requires=[
        'pychrome >= 0.2.2',
        'requests >= 2.19.1',
        'selenium >= 3.13.0'
    ]
)
